print(": Registering basic commands")

class ExtendedParseStepInfo(gdb.Parameter):
	"""Controls whether to display parser stack and input preview on stepping the parse."""
	def __init__(self):
		super(ExtendedParseStepInfo, self).__init__("hammer-extended-parse-step-info", gdb.COMMAND_OBSCURE, gdb.PARAM_BOOLEAN)
		self.show_doc = "Show parser stack and input preview after hammer-parse-step:"
		#self.set_doc = "Show parser stack and input preview after hammer-parse-step:"
		self.value = True
		print(":: hammer-extended-parse-step-info")

ExtendedParseStepInfo()

class HammerParserBacktrace(gdb.Command):
	def __init__(self):
		super(HammerParserBacktrace, self).__init__ ("hammer-parser-backtrace", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-backtrace")

	def invoke(self, arg, from_tty):
		parserstack = top_level_parse.peek_parserstack().p_stack
		args = gdb.string_to_argv(arg)
		if len(args) < 1:
			maxsize = len(parserstack)
		else:
			try:
				maxsize = int(args[0])
				if maxsize < 1:
					raise ValueError
			except ValueError:
				maxsize = len(parserstack)
				print("Argument must be a positive integer")

		print("[" + str(hex(top_level_parse.h_do_parse_parser.address)) + "] " + top_level_parse.h_do_parse_parser.name + " [current]") #TODO: GUI widget should reflect this
		print(" ")
		depth = min(len(parserstack), maxsize)
		if depth > 0: # if stack not empty
			# unsure what the idiomatic python is for handling negative indices starting with -1,
			# but this addition is to avoid off-by-one errors
			index = -(depth+1)
			for p in parserstack[-1:index:-1]:
				print("[" + str(hex(p.address)) + "] " + p.name) # TODO: errors in perform_lowlevel_parse, if p.name is None
			if depth < len(parserstack):
				print("[...]")

HammerParserBacktrace()

class HammerParserMemUse(gdb.Command):
	def __init__(self):
		super(HammerParserMemUse, self).__init__("hammer-parser-mem-use", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-mem-use")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)
		if len(args) < 1:
			print("Usage: hammer-parser-mem-use <address>")
			return

		parser_addr = args[0]
		try:
			parser_addr_int = int(parser_addr, 16)
			parser_obj = top_level_parse.parser_by_address(parser_addr_int)
			if parser_obj is not None:
				print(parser_obj.bytes_used)
		except ValueError:
			print("Address needs to be a hexadecimal number")

HammerParserMemUse()

class HammerParserMemUseName(gdb.Command):
	def __init__(self):
		super(HammerParserMemUseName, self).__init__("hammer-parser-mem-use-name", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-mem-use-name")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)
		if len(args) < 1:
			print("Usage: hammer-parser-mem-use-name <name>")
			return

		parser_name = args[0]
		parser_objs = top_level_parse.parsers_by_name(parser_name)
		if parser_objs is not None:
			for p in parser_objs:
				print((p.name, hex(p.address), p.bytes_used))

HammerParserMemUseName()

class HammerParserTopSingleArenaMem(gdb.Command):
	def __init__(self):
		super(HammerParserTopSingleArenaMem, self).__init__("hammer-parser-top-single-arena-mem", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-top-single-arena-mem")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)

		p = top_level_parse.get_parser_top_per_arena_mem()
		print((p.name, hex(p.address), p.bytes_used))

HammerParserTopSingleArenaMem()

class HammerParserTopTotalArenaMem(gdb.Command):
	def __init__(self):
		super(HammerParserTopTotalArenaMem, self).__init__("hammer-parser-top-total-arena-mem", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-top-total-arena-mem")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)

		p = top_level_parse.get_parser_top_total_arena_mem()
		print((p.name, hex(p.address), p.bytes_used))
		total_mem_use = p.get_arenasum()
		print("Total: " + str(total_mem_use) + " bytes")

HammerParserTopTotalArenaMem()

# TODO: average memory use, per arena and total

class HammerParserPreviewInput(gdb.Command):
	def __init__(self):
		super(HammerParserPreviewInput, self).__init__("hammer-parser-preview-input", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-preview-input")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)

		print(top_level_parse.input_chunk)

HammerParserPreviewInput()

class HammerParserAverageMem(gdb.Command):
	def __init__(self):
		super(HammerParserAverageMem, self).__init__("hammer-parser-average-mem", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-average-mem")

	def invoke(self, arg, from_tty):
		args = gdb.string_to_argv(arg)

		mem = top_level_parse.get_avg_mem_use_per_arena()
		print("Bytes used on average in each arena:")
		print(mem)

HammerParserAverageMem()

class HammerParserCurrentEnv(gdb.Command):
	def __init__(self):
		super(HammerParserCurrentEnv, self).__init__("hammer-parser-current-env", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-current-env")

	def invoke(self, arg, from_tty):
		p = top_level_parse.h_do_parse_parser
		p_env = top_level_parse.parser_decombinator.decompose_parser(p, top_level_parse) #TODO: parser -> env mapping function in top_level_parse
		print(type(p_env).__name__ + " - " + str(p_env)) # TODO: consistency with GUI

HammerParserCurrentEnv()

class HammerParserDumpMemoryStats(gdb.Command):
	def __init__(self):
		super(HammerParserDumpMemoryStats, self).__init__("hammer-parser-dump-memory-stats", gdb.COMMAND_OBSCURE)
		print(":: hammer-parser-dump-memory-stats")

	def invoke(self, arg, from_tty):
		stats = top_level_parse.dump_memory_stats()
		print(stats)

HammerParserDumpMemoryStats()
