class HPackratParseBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()
		for val in block: # GDB, why did you take away [] for gdb.Block?
			if val.name == 'parser':
				parser = int(val.value(frame))
		top_level_parse.enter_h_packrat_parse(parser)
		return False

class HPackratParseRetBreakpoint(gdb.Breakpoint):
	def stop(self):
		top_level_parse.return_from_h_packrat_parse()
		return False

# TODO: frame.older() allows accessing the caller frame. decision logic about whether to call first_h_do_parse_after_packrat_parse()
# could be moved here
class HDoParseBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()
		retval = False
		for val in block:
			if val.name == 'parser':
				parser = int(val.value(frame))
			if val.name == 'state':
				#TODO: rename these variables to make it clear they're pointers
				state = int(val.value(frame))
				state_obj = val.value(frame)
				index = val.value(frame).dereference()['input_stream']['index']
				input_ptr = val.value(frame).dereference()['input_stream']['input']
				# If you want to printf debug the parse state
				#print(val.value(frame).dereference())
			if val.name == 'arena':
				arena = int(val.value(frame))
		top_level_parse.enter_h_do_parse(state, None, parser)

		input_chunk = input_ptr + index
		#print(input_chunk.string('ascii','backslashreplace',10))
		if int(input_ptr) != 0: # XXX: checking gdb values for 0
			top_level_parse.set_input_chunk(input_chunk.string('UTF-8','replace',32)) # XXX: breaks after a filter fails to execute

		# Check if we need to stop after a number of steps
		step_counter = gdb.convenience_variable("hammer_step_counter")
		if step_counter is not None and step_counter > 0:
			step_counter -= 1
			if step_counter == 0:
				gdb.set_convenience_variable("hammer_step_counter", None) # unset step counter
				retval = True
			else:
				gdb.set_convenience_variable("hammer_step_counter", step_counter)
		#else:
			# retval remains False

		# Check if we need to stop at a position
		stop_pos = gdb.convenience_variable("hammer_stop_pos")
		if stop_pos is not None and stop_pos <= index:
			retval = True
			gdb.set_convenience_variable("hammer_stop_pos", None)
			print("Requested stop position: " + str(stop_pos) + " Stopped at: " + str(index))
		#return False
		return retval

class PerformLowLevelParseBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()
		#retval = False
		for val in block:
			if val.name == 'parser':
				parser = int(val.value(frame))
		top_level_parse.enter_perform_lowlevel_parse(parser)

		#return retval
		return False

class PerformLowLevelParseRetBreakpoint(gdb.Breakpoint):
	def stop(self):
		top_level_parse.return_from_perform_lowlevel_parse()
		return False

class ParserVirtualBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()
		# function name is parse_*
		# we extract the second part
		# This is pretty much the same as naming it based on which parse_* function is called, so long as foo_vt contains parse_foo , and not e.g. parse_bar
		parser_type = frame.function().name.split("_")[1]
		parser_name = "(Unnamed " + parser_type + ")"
		top_level_parse.parse_virtual(parser_name)
		return False

class HArenaMallocRawBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()

		for val in block:
			if val.name == 'size':
				alloc_size = int(val.value(frame))

		top_level_parse.enter_h_arena_malloc_raw(alloc_size)

		return False

class HRuleBreakpoint(gdb.Breakpoint):
	def stop(self):
		frame = gdb.selected_frame()
		block = frame.block()
		parser_type = gdb.lookup_type("HParser").pointer()

		for p in block:
			# GDB (Debian 10.1-2) with python 3.9 crashes when trying to compare these two, but doing this would filter out locals that aren't parsers
			#if p.type == parser_type:
			top_level_parse.parser_objs[int(p.value(frame))] = Parser(p.name, int(p.value(frame)))

